@@ -1,12 +1,14 @@
 import asyncio
 import json
 from copy import deepcopy
+from typing import Callable
+
 import redis
 import datetime
 
 from django.db.models import Model as DjangoModel
 
-from django_redis_orm.utils import check_types
+from python_redis_orm.utils import check_types
 
 
 class RedisField:
@@ -290,41 +292,81 @@ class RedisList(RedisJson):
 
 class RedisRoot:
 
-    def __init__(self, redis_instance=None, prefix='redis_test', ignore_deserialization_errors=True,
+    def __init__(self, connection_pool=None, prefix='redis_test', ignore_deserialization_errors=True,
                  save_consistency=False, economy=False):
-        self.registered_models = []
+        self.registered_models = {}
         if type(prefix) == str:
             self.prefix = prefix
         else:
             print(f'Prefix {prefix} is type of {type(prefix)}, allowed only str, using default prefix "redis_test"')
             self.prefix = 'redis_test'
+        self.connection_pool = self._get_connection_pool(connection_pool)
         self.ignore_deserialization_errors = ignore_deserialization_errors
         self.save_consistency = save_consistency
         self.economy = economy
-        if redis_instance:
-            self.redis_instance = redis_instance
+        self.check_cache()
+
+    @property
+    def redis_instance(self):
+        redis_instance = redis.Redis(connection_pool=self.connection_pool)
+        return redis_instance
+
+    def _get_redis_cache_existing_data(self):
+        redis_models_with_redis_dicts = {}
+        for redis_model, to_cache_in in self.registered_models.items():
+            now = datetime.datetime.now()
+            if to_cache_in >= now:
+                meta = redis_model.__model_data__['meta']
+                cache_conf = {}
+                enabled = False
+                if 'cache' in meta.keys():
+                    cache_conf = meta['cache']
+                    if cache_conf['enabled']:
+                        enabled = True
+                if enabled:
+                    ttl = cache_conf['ttl']
+                    instances = self._get_all_model_instances(redis_model)
+                    redis_models_with_redis_dicts[redis_model] = instances
+                    now = datetime.datetime.now()
+                    self.registered_models[redis_model] = now + datetime.timedelta(seconds=ttl)
+        return redis_models_with_redis_dicts
+
+    def check_cache(self):
+        redis_models_with_redis_dicts = self._get_redis_cache_existing_data()
+        for redis_model, redis_dicts in redis_models_with_redis_dicts.items():
+            meta = redis_model.__model_data__['meta']
+            cache_conf = meta['cache']
+            django_model = meta['django_model']
+            cache_func = cache_conf['cache_func']
+            cache_func(redis_model, redis_dicts, django_model)
+
+    def _get_connection_pool(self, connection_pool):
+        if isinstance(connection_pool, redis.ConnectionPool):
+            connection_pool.connection_kwargs['decode_responses'] = True
+            self.connection_pool = connection_pool
         else:
-            print(f'{self.__class__.__name__}: No redis_instance provided, trying default config...')
+            print(f'{self.__class__.__name__}: No connection_pool provided, trying default config...')
             default_host = 'localhost'
             default_port = 6379
             default_db = 0
             try:
-                redis_instance = redis.StrictRedis(
+                connection_pool = redis.ConnectionPool(
                     decode_responses=True,
                     host=default_host,
                     port=default_port,
                     db=default_db,
                 )
-                self.redis_instance = redis_instance
+                self.connection_pool = connection_pool
             except BaseException as ex:
                 raise Exception(
-                    f'Default config ({default_host}:{default_port}, db={default_db}) failed, please provide redis_instance to {self.__class__.__name__}')
+                    f'Default config ({default_host}:{default_port}, db={default_db}) failed, please provide connection_pool to {self.__class__.__name__}')
+        return self.connection_pool
 
     def register_models(self, models_list):
         for model in models_list:
             if issubclass(model, RedisModel):
                 if model not in self.registered_models:
-                    self.registered_models.append(model)
+                    self.registered_models[model] = datetime.datetime.now()
             else:
                 raise Exception(f'{model.__name__} class is not RedisModel')
 
@@ -516,8 +558,8 @@ class RedisRoot:
         instances = self._get_instances_from_instances_with_allowed(instances_with_allowed)
         return instances
 
-    def get(self, model, return_dict=False, **kwargs):
-        instances = self._get_all_model_instances(model, kwargs)
+    def get(self, model, return_dict=False, **filters):
+        instances = self._get_all_model_instances(model, filters)
         return self._return_with_format(instances, return_dict)
 
     def order(self, instances, field_name):
@@ -629,7 +671,6 @@ class RedisModel:
     def __init__(self, redis_root=None, **kwargs):
         self.__model_data__ = {
             'redis_root': None,
-            'redis_instance': None,
             'name': None,
             'fields': None,
             'meta': {}
@@ -637,11 +678,10 @@ class RedisModel:
 
         if isinstance(redis_root, RedisRoot):
             self.__model_data__['redis_root'] = redis_root
-            self.__model_data__['redis_instance'] = redis_root.redis_instance
             self.__model_data__['name'] = self.__class__.__name__
             if self.__class__ != RedisModel:
-                self.__model_data__['redis_root'].register_models([self.__class__])
                 self._renew_fields()
+                self.__model_data__['redis_root'].register_models(self.__class__)
                 self._fill_fields_values(kwargs)
 
         else:
@@ -651,9 +691,56 @@ class RedisModel:
         check_types(ttl, (int, float))
         return ttl
 
+    def _check_meta_cache(self, user_cache_conf):
+        name = self.__model_data__['name']
+        redis_root = self.__model_data__['redis_root']
+
+        def default_cache_func(*args, **kwargs):
+            return None
+
+        cache_conf = {
+            'enabled': False,
+            'ttl': 60 * 1,
+            'cache_func': default_cache_func,
+            'prefix': f'{redis_root.prefix}-{name}-cache',
+        }
+
+        if type(user_cache_conf) == dict:
+            if 'enabled' in user_cache_conf.keys():
+                if type(user_cache_conf['enabled']) == bool:
+                    cache_conf['enabled'] = user_cache_conf['enabled']
+                else:
+                    raise Exception(f'{name} -> Meta -> cache -> enabled must be bool')
+            if 'ttl' in user_cache_conf.keys():
+                if type(user_cache_conf['ttl']) == int:
+                    cache_conf['ttl'] = user_cache_conf['ttl']
+                else:
+                    raise Exception(f'{name} -> Meta -> cache -> ttl must be int')
+            if 'django_model' in user_cache_conf.keys():
+                if issubclass(user_cache_conf['django_model'], DjangoModel):
+                    cache_conf['django_model'] = user_cache_conf['django_model']
+                else:
+                    raise Exception(f'{name} -> Meta -> cache -> django_model must be subclass of django.db.models Model')
+            if 'cache_func' in user_cache_conf.keys():
+                if type(user_cache_conf['cache_func']) == Callable:
+                    cache_conf['cache_func'] = user_cache_conf['cache_func']
+                else:
+                    raise Exception(
+                        f'{name} -> Meta -> cache -> cache_func must be callable function')
+            if 'prefix' in user_cache_conf.keys():
+                if type(user_cache_conf['prefix']) == str:
+                    cache_conf['prefix'] = user_cache_conf['prefix']
+                else:
+                    raise Exception(f'{name} -> Meta -> cache -> prefix must be str')
+        else:
+            raise Exception(f'{name} -> Meta -> cache must be dict')
+
+        return cache_conf
+
     def _set_meta(self, meta_fields):
         allowed_meta_fields_with_check_functions = {
-            'ttl': self._check_meta_ttl
+            'ttl': self._check_meta_ttl,
+            'cache': self._check_meta_cache,
         }
         for field_name, field_value in meta_fields.items():
             if field_name in allowed_meta_fields_with_check_functions.keys():
@@ -662,10 +749,12 @@ class RedisModel:
                     self.__model_data__['meta'][field_name] = cleaned_value
 
     def _get_initial_model_field(self, field_name):
-        if field_name in self.__class__.__dict__.keys():
+        name = self.__model_data__['name']
+        fields = self.__model_data__['fields']
+        if field_name in fields.keys():
             return deepcopy(self.__class__.__dict__[field_name])
         else:
-            raise Exception(f'{self.__class__} has no field {field_name}')
+            raise Exception(f'{name} has no field {field_name}')
 
     def _renew_fields(self):
         class_fields = self.__class__.__dict__.copy()
@@ -697,7 +786,7 @@ class RedisModel:
                 field_ttl = meta['ttl']
         return field_ttl
 
-    def _get_fields_with_model_key(self):
+    def _serialize_data(self, full=True):
         redis_root = self.__model_data__['redis_root']
         name = self.__model_data__['name']
         fields = self.__model_data__['fields']
@@ -708,19 +797,22 @@ class RedisModel:
             if not field_name.startswith('__'):
                 try:
                     cleaned_value = field.clean()
-                    field_ttl = self.get_field_ttl(field)
-                    cleaned_fields[model_key + f':{field_name}'] = {
-                        'value': cleaned_value,
-                        'ttl': field_ttl
-                    }
+                    if full:
+                        field_ttl = self.get_field_ttl(field)
+                        cleaned_fields[model_key + f':{field_name}'] = {
+                            'value': cleaned_value,
+                            'ttl': field_ttl
+                        }
+                    else:
+                        cleaned_fields[field_name] = cleaned_value
                 except BaseException as ex:
                     raise Exception(f'{ex} ({name} -> {field_name})')
         fields_with_model_key = cleaned_fields
         return fields_with_model_key
 
     def _set_fields(self):
-        field_to_write = self._get_fields_with_model_key()
-        redis_instance = self.__model_data__['redis_instance']
+        field_to_write = self._serialize_data()
+        redis_instance = self.__model_data__['redis_root'].redis_instance
         for key, field_data in field_to_write.items():
             redis_instance.set(key, field_data['value'], ex=field_data['ttl'])
 
@@ -743,3 +835,30 @@ class RedisModel:
             deserialized_instance = redis_root.get(self.__class__, id=self.id.value)[0]
         return deserialized_instance
 
+    def set(self, **fields_with_values):
+        name = self.__model_data__['name']
+        fields = self.__model_data__['fields']
+        meta = self.__model_data__['meta']
+        for field_name, value in fields_with_values.items():
+            if field_name in fields.keys():
+                field = fields[field_name]
+                field.value = value
+                return field.value
+            elif field_name in meta.keys():
+                meta[field_name] = value
+                return meta[field_name]
+            else:
+                raise Exception(f'{name} has no field {field_name}')
+
+    def get(self, field_name):
+        name = self.__model_data__['name']
+        fields = self.__model_data__['fields']
+        meta = self.__model_data__['meta']
+        if field_name in fields.keys():
+            field = fields[field_name]
+            return field.value
+        elif field_name in meta.keys():
+            return meta[field_name]
+        else:
+            raise Exception(f'{name} has no field {field_name}')
+
