@@ -56,33 +56,33 @@ class ListCheckModel(RedisModel):
     redis_list = RedisList()
 
 
-class DjangoForeignKeyModel(RedisModel):
-    foreign_key = RedisDjangoForeignKey(model=Proxy)
+# class DjangoForeignKeyModel(RedisModel):
+#     foreign_key = RedisDjangoForeignKey(model=Proxy)
 
 
-def get_redis_instance(connection_pool=None):
+def get_connection_pool():
     REDIS_HOST = 'localhost'
     REDIS_PORT = 6379
-    redis_instance = redis.Redis(
+    connection_pool = redis.ConnectionPool(
         decode_responses=True,
         host=REDIS_HOST,
         port=REDIS_PORT,
         db=0,
-        connection_pool=connection_pool
     )
-    return redis_instance
+    return connection_pool
 
 
-def clean_db_after_test(redis_instance, prefix):
+def clean_db_after_test(connection_pool, prefix):
+    redis_instance = redis.Redis(connection_pool=connection_pool)
     for key in redis_instance.scan_iter(f'{prefix}:*'):
         redis_instance.delete(key)
 
 
-def basic_test(redis_instance, prefix):
+def basic_test(connection_pool, prefix):
     try:
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
+            connection_pool=connection_pool,
             ignore_deserialization_errors=True
         )
         redis_root.register_models([
@@ -108,14 +108,14 @@ def basic_test(redis_instance, prefix):
                     have_exception = True
     except BaseException as ex:
         have_exception = True
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def auto_reg_test(redis_instance, prefix):
+def auto_reg_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     task_challenge_1 = TaskChallenge(
@@ -127,7 +127,7 @@ def auto_reg_test(redis_instance, prefix):
         have_exception = False
     except BaseException as ex:
         have_exception = True
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
@@ -149,10 +149,10 @@ def no_redis_instance_test(*args, **kwargs):
     return have_exception
 
 
-def choices_test(redis_instance, prefix):
+def choices_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     task_challenge_1 = TaskChallenge(
@@ -165,14 +165,14 @@ def choices_test(redis_instance, prefix):
         have_exception = True
     except BaseException as ex:
         have_exception = False
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def order_test(redis_instance, prefix):
+def order_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     for i in range(3):
@@ -188,14 +188,14 @@ def order_test(redis_instance, prefix):
             have_exception = False
     except BaseException as ex:
         pass
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def filter_test(redis_instance, prefix):
+def filter_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -214,14 +214,14 @@ def filter_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def update_test(redis_instance, prefix):
+def update_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -238,14 +238,14 @@ def update_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def functions_like_defaults_test(redis_instance, prefix):
+def functions_like_defaults_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = False
@@ -257,14 +257,14 @@ def functions_like_defaults_test(redis_instance, prefix):
     except BaseException as ex:
         pass
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def redis_foreign_key_test(redis_instance, prefix):
+def redis_foreign_key_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -286,14 +286,14 @@ def redis_foreign_key_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-# def redis_django_foreign_key_test(redis_instance, prefix):
+# def redis_django_foreign_key_test(connection_pool, prefix):
 #     redis_root = RedisRoot(
 #         prefix=prefix,
-#         redis_instance=redis_instance,
+#         connection_pool=connection_pool,
 #         ignore_deserialization_errors=True
 #     )
 #     have_exception = True
@@ -315,14 +315,14 @@ def redis_foreign_key_test(redis_instance, prefix):
 #     except BaseException as ex:
 #         print(ex)
 #
-#     clean_db_after_test(redis_instance, prefix)
+#     clean_db_after_test(connection_pool, prefix)
 #     return have_exception
 
 
-def delete_test(redis_instance, prefix):
+def delete_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -343,14 +343,14 @@ def delete_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def ttl_test(redis_instance, prefix):
+def ttl_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -371,14 +371,14 @@ def ttl_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def save_consistency_test(redis_instance, prefix):
+def save_consistency_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True,
         save_consistency=True,
     )
@@ -400,14 +400,14 @@ def save_consistency_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def meta_ttl_test(redis_instance, prefix):
+def meta_ttl_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True,
     )
     have_exception = True
@@ -426,17 +426,17 @@ def meta_ttl_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def economy_test(redis_instance, prefix):
+def economy_test(connection_pool, prefix):
     have_exception = True
     try:
 
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
+            connection_pool=connection_pool,
             ignore_deserialization_errors=True,
             economy=True
         )
@@ -449,11 +449,10 @@ def economy_test(redis_instance, prefix):
             redis_root.update(TaskChallenge, task_challenge_1, account_checks_count=1)
         ended_in_economy = datetime.datetime.now()
         economy_time = (ended_in_economy - started_in_economy).total_seconds()
-        clean_db_after_test(redis_instance, prefix)
+        clean_db_after_test(connection_pool, prefix)
 
         redis_root = RedisRoot(
             prefix=prefix,
-            redis_instance=redis_instance,
             ignore_deserialization_errors=True,
             economy=False
         )
@@ -466,7 +465,7 @@ def economy_test(redis_instance, prefix):
             redis_root.update(TaskChallenge, task_challenge_1, account_checks_count=1)
         ended_in_no_economy = datetime.datetime.now()
         no_economy_time = (ended_in_no_economy - started_in_no_economy).total_seconds()
-        clean_db_after_test(redis_instance, prefix)
+        clean_db_after_test(connection_pool, prefix)
         economy_percent = round((no_economy_time / economy_time - 1) * 100, 2)
         economy_symbol = ('+' if economy_percent > 0 else '')
         print(f'Economy gives {economy_symbol}{economy_percent}% efficiency')
@@ -475,14 +474,14 @@ def economy_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def dict_test(redis_instance, prefix):
+def dict_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -502,14 +501,14 @@ def dict_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def list_test(redis_instance, prefix):
+def list_test(connection_pool, prefix):
     redis_root = RedisRoot(
         prefix=prefix,
-        redis_instance=redis_instance,
+        connection_pool=connection_pool,
         ignore_deserialization_errors=True
     )
     have_exception = True
@@ -526,29 +525,23 @@ def list_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
-def async_test(redis_instance, prefix):
+def async_test(connection_pool, prefix):
     have_exception = True
     try:
 
         async def async_task(data_count, use_async=True):
-            redis_roots = []
-            connection_pool = redis.ConnectionPool(host='localhost', port=6379, db=0, decode_responses=True)
-            for i in range(data_count):
-                redis_instance = get_redis_instance(connection_pool)
-                redis_root = RedisRoot(
-                    prefix=prefix,
-                    redis_instance=redis_instance,
-                    ignore_deserialization_errors=True
-                )
-                redis_roots.append(redis_root)
+            redis_root = RedisRoot(
+                prefix=prefix,
+                connection_pool=connection_pool,
+                ignore_deserialization_errors=True
+            )
 
-            async def create_list(redis_roots, redis_root_index, list_length):
+            async def create_list(list_length):
                 some_list = [random.choice('ABCDEF') for i in range(list_length)]
-                redis_root = redis_roots[redis_root_index]
                 list_check_model_instance = ListCheckModel(
                     redis_root=redis_root,
                     redis_list=some_list
@@ -558,18 +551,16 @@ def async_test(redis_instance, prefix):
                     'id': list_check_model_instance['id']
                 }
 
-            async def get_object_existence(redis_roots, redis_root_index, lists_with_ids):
+            async def get_object_existence(lists_with_ids):
                 exists = False
-                redis_root = redis_roots[redis_root_index]
                 objects = redis_root.get(ListCheckModel, redis_list=lists_with_ids['list'])
                 if objects:
                     if len(objects) == 1:
                         exists = objects[0]['id'] == lists_with_ids['id']
                 return exists
 
-            async def update_list(redis_roots, redis_root_index, lists_with_ids):
+            async def update_list(lists_with_ids):
                 result = False
-                redis_root = redis_roots[redis_root_index]
                 objects = redis_root.get(ListCheckModel, redis_list=lists_with_ids['list'])
                 if objects:
                     if len(objects) == 1:
@@ -583,9 +574,8 @@ def async_test(redis_instance, prefix):
                                 result = updated_objects[0]['id'] == lists_with_ids['id']
                 return result
 
-            async def delete_list(redis_roots, redis_root_index, id):
+            async def delete_list(id):
                 result = False
-                redis_root = redis_roots[redis_root_index]
                 objects = redis_root.get(ListCheckModel, id=id)
                 if objects:
                     if len(objects) == 1:
@@ -599,13 +589,13 @@ def async_test(redis_instance, prefix):
             async def create_lists(lists_count, use_async=True):
                 if use_async:
                     async_create_lists_tasks = [
-                        create_list(redis_roots, i, i + 100)
+                        create_list(i + 100)
                         for i in range(lists_count)
                     ]
                     list_of_lists_with_ids = await asyncio.gather(*async_create_lists_tasks)
                 else:
                     list_of_lists_with_ids = [
-                        await create_list(redis_roots, i, i + 100)
+                        await create_list(i + 100)
                         for i in range(lists_count)
                     ]
                 return list_of_lists_with_ids
@@ -613,13 +603,13 @@ def async_test(redis_instance, prefix):
             async def get_objects_existence(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_get_object_by_list_tasks = [
-                        get_object_existence(redis_roots, i, lists_with_ids)
+                        get_object_existence(lists_with_ids)
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                     list_of_results = await asyncio.gather(*async_get_object_by_list_tasks)
                 else:
                     list_of_results = [
-                        await get_object_existence(redis_roots, i, lists_with_ids)
+                        await get_object_existence(lists_with_ids)
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
@@ -627,13 +617,13 @@ def async_test(redis_instance, prefix):
             async def update_lists(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_update_list_tasks = [
-                        update_list(redis_roots, i, lists_with_ids)
+                        update_list(lists_with_ids)
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                     list_of_results = await asyncio.gather(*async_update_list_tasks)
                 else:
                     list_of_results = [
-                        await update_list(redis_roots, i, lists_with_ids)
+                        await update_list(lists_with_ids)
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
@@ -641,13 +631,13 @@ def async_test(redis_instance, prefix):
             async def delete_lists(list_of_lists_with_ids, use_async=True):
                 if use_async:
                     async_delete_list_tasks = [
-                        delete_list(redis_roots, i, lists_with_ids['id'])
+                        delete_list(lists_with_ids['id'])
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                     list_of_results = await asyncio.gather(*async_delete_list_tasks)
                 else:
                     list_of_results = [
-                        await delete_list(redis_roots, i, lists_with_ids)
+                        await delete_list(lists_with_ids)
                         for i, lists_with_ids in enumerate(list_of_lists_with_ids)
                     ]
                 return list_of_results
@@ -680,12 +670,12 @@ def async_test(redis_instance, prefix):
     except BaseException as ex:
         print(ex)
 
-    clean_db_after_test(redis_instance, prefix)
+    clean_db_after_test(connection_pool, prefix)
     return have_exception
 
 
 def run_tests():
-    redis_instance = get_redis_instance()
+    connection_pool = get_connection_pool()
     tests = [
         basic_test,
         auto_reg_test,
@@ -712,7 +702,7 @@ def run_tests():
     for i, test in enumerate(tests):
         print(f'Starting {int(i + 1)} test: {test.__name__.replace("_", " ")}')
         test_started_in = datetime.datetime.now()
-        result = not test(redis_instance, test.__name__)
+        result = not test(connection_pool, test.__name__)
         test_ended_in = datetime.datetime.now()
         test_time = (test_ended_in - test_started_in).total_seconds()
         print(f'{result = } / {test_time}s\n')
